package api

import (
	"encoding/json"
	"net/http"
	"time"

	"github.com/ethereum/go-ethereum/common"
)

func rpcHandle(w http.ResponseWriter, r *http.Request) {
	req := &jsonrpcMessage{}
	resp := &jsonrpcMessage{}

	if r.Method != "POST" {
		w.Write([]byte("method not allowed"))
		w.WriteHeader(http.StatusMethodNotAllowed)
		return
	}

	w.Header().Set("Content-Type", "application/json")
	contentType := r.Header.Get("Content-Type")
	if contentType != "application/json" {
		resp.Error = &jsonError{
			Code:    -32600,
			Message: "invalid Content-Type header",
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

	err := json.NewDecoder(r.Body).Decode(req)
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32603,
			Message: err.Error(),
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

	if req.Version != "2.0" {
		resp.Error = &jsonError{
			Code:    -32600,
			Message: "invalid jsonrpc version",
		}
		resp.Version = "2.0"
		_ = json.NewEncoder(w).Encode(resp)
		return
	}
	resp.Version = req.Version
	resp.ID = req.ID

	switch req.Method {
	case "getWithdrawProofs":
		getWithdrawProofs(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
		return
	case "getPendingWorkload":
		getPendingWorkload(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
		return
	case "getDailyMerkleNodes":
		getDailyMerkleNodes(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
	case "getDailyMerkleSumNodes":
		getDailyMerkleSumNodes(req.Params, resp)
		_ = json.NewEncoder(w).Encode(resp)
	default:
		resp.Error = &jsonError{
			Code:    -32601,
			Message: "method not found",
		}
		_ = json.NewEncoder(w).Encode(resp)
		return
	}

}

func getWithdrawProofs(params []byte, resp *jsonrpcMessage) {
	paramList := make([]string, 0)
	err := json.Unmarshal(params, &paramList)
	if err != nil || len(paramList) < 1 || len(paramList) > 2 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	if !common.IsHexAddress(paramList[0]) {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	var date string
	if len(paramList) > 1 {
		_, err = time.Parse("2006-01-02", paramList[1])
		if err != nil {
			resp.Error = &jsonError{
				Code:    -32602,
				Message: "invalid params",
			}
			return
		}
		date = paramList[1]
	}

	amount, proofs := validator.GetMerkleProof(common.HexToAddress(paramList[0]), date)

	temp := map[string]interface{}{
		"amount": amount,
		"proofs": proofs,
	}

	resp.Result, _ = json.Marshal(temp)
	return
}

func getPendingWorkload(params []byte, resp *jsonrpcMessage) {
	addressList := make([]string, 0)
	err := json.Unmarshal(params, &addressList)
	if err != nil || len(addressList) != 1 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	if !common.IsHexAddress(addressList[0]) {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	workload, globalWorkload := validator.GetPendingWorkload(common.HexToAddress(addressList[0]))
	temp := map[string]interface{}{
		"workload":        workload,
		"global_workload": globalWorkload,
	}

	resp.Result, _ = json.Marshal(temp)
	return
}

func getDailyMerkleNodes(params []byte, resp *jsonrpcMessage) {
	// date string, depth int, rootHash common.Hash
	paramList := make([]interface{}, 0)
	err := json.Unmarshal(params, &paramList)
	if err != nil || len(paramList) < 1 || len(paramList) > 3 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	var date string
	var depth = 1
	var rootHash common.Hash

	_, err = time.Parse("2006-01-02", paramList[0].(string))
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}
	date = paramList[0].(string)

	if len(paramList) >= 2 {
		_depth, ok := paramList[1].(float64)
		if !ok {
			resp.Error = &jsonError{
				Code:    -32602,
				Message: "invalid params",
			}
			return
		}
		depth = int(uint(_depth))
	}

	if len(paramList) >= 3 {
		rootHash = common.HexToHash(paramList[2].(string))
	}

	nodes := validator.GetDailyMerkleNodes(date, depth, rootHash)
	resp.Result, _ = json.Marshal(nodes)
}

func getDailyMerkleSumNodes(params []byte, resp *jsonrpcMessage) {
	// date string, depth int, rootHash common.Hash
	paramList := make([]interface{}, 0)
	err := json.Unmarshal(params, &paramList)
	if err != nil || len(paramList) < 1 || len(paramList) > 3 {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}

	var date string
	var depth = 1
	var rootHash common.Hash

	_, err = time.Parse("2006-01-02", paramList[0].(string))
	if err != nil {
		resp.Error = &jsonError{
			Code:    -32602,
			Message: "invalid params",
		}
		return
	}
	date = paramList[0].(string)

	if len(paramList) >= 2 {
		_depth, ok := paramList[1].(float64)
		if !ok {
			resp.Error = &jsonError{
				Code:    -32602,
				Message: "invalid params",
			}
			return
		}
		depth = int(uint(_depth))
	}

	if len(paramList) >= 3 {
		rootHash = common.HexToHash(paramList[2].(string))
	}

	nodes, vals := validator.GetDailyMerkleSumNodes(date, depth, rootHash)
	resp.Result, _ = json.Marshal(map[string]interface{}{
		"nodes":  nodes,
		"values": vals,
	})
}
